#!/usr/bin/python -u
# coding: utf-8

#--------------------------------------------------------------------
# Fichero:  imprime-preferencias.py
# Objetivo: Crear un PDF con todas las preferencias
# Fecha:    X.6.11.2019
# Autor:    Pedro Reina
# Licencia: Dominio público
#           https://creativecommons.org/publicdomain/zero/1.0/deed.es
#--------------------------------------------------------------------

#------------------------------------------------
# Variables del programa
#------------------------------------------------

# El archivo con la base de datos
BaseDato = 'maxifeli.db'

# Archivos TTF con las fuentes
FuenteRegular = '/usr/share/fonts/truetype/droid/DroidSans.ttf'
FuenteNegrita = '/usr/share/fonts/truetype/droid/DroidSans-Bold.ttf'
FuenteMono = '/usr/share/fonts/truetype/droid/DroidSansMono.ttf'

# Tamaños de las letras
TamanoGrande = 14
TamanoPequeno = 10

# Grosor de las líneas
Grosor = 0.5

# El PDF final
Documento = 'preferencias.pdf'

# Los márgenes de las páginas, en centímetros
Margen = 1.2

# La distancia entre líneas de texto, en centímetros
Interlineado = 0.6

#------------------------------------------------
# Módulos necesarios
#------------------------------------------------

# Another Python SQLite Wrapper
import apsw

# Para averiguar la fecha y la hora
import datetime

# Configuración de ReportLab
import reportlab.rl_config

# Creación de PDF con ReportLab
from reportlab.pdfgen import canvas
from reportlab.lib.units import cm
from reportlab.lib.pagesizes import A4
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.platypus import Table, TableStyle
from reportlab.lib import colors

#------------------------------------------------
# Variables globales
#------------------------------------------------

# La lista con los nombres y usuarios
ListaNombreUsuario = []

#------------------------------------------------
# Funciones
#------------------------------------------------

#------------------------------------------------
# Dice la fecha y la hora
def FechaHora():
    Fecha = datetime.datetime.now().strftime('%Y-%m-%d')
    Hora = datetime.datetime.now().strftime('%H:%M:%S')
    return Fecha + u' ◊ ' + Hora

#------------------------------------------------
# Devuelve la lista de nombres y usuarios de la base de datos
def LeeNombreUsuario(Conexion):
    Lista = []
    Consulta = ('SELECT nombre, usuario FROM acceso')

    Cursor = Conexion.cursor()
    for Dato in Cursor.execute(Consulta):
        Lista.append((Dato[0], Dato[1]))

    return Lista

#------------------------------------------------
# Devuelve un diccionario de preferencias de la base de datos
def LeePreferencias(Conexion):
    Diccionario = {}

    # Por defecto no hay preferencias establecidas
    for NombreUsuario in ListaNombreUsuario:
        Activo = NombreUsuario[1]
        for NombreUsuarioPasivo in ListaNombreUsuario:
            Pasivo = NombreUsuarioPasivo[1]
            if Activo == Pasivo: Diccionario[(Activo, Pasivo)] = 'x'
            else:                Diccionario[(Activo, Pasivo)] = ''

    # Consultamos las preferencias que hay en la base de datos
    Consulta = ('SELECT activo, pasivo, cantidad FROM preferencia')
    Cursor = Conexion.cursor()
    for Dato in Cursor.execute(Consulta):
        Activo = Dato[0]
        Pasivo = Dato[1]
        Cantidad = Dato[2]
        Diccionario[(Activo, Pasivo)] = str(Cantidad)

    # Devolvemos el diccionario creado
    return Diccionario

#------------------------------------------------
# Genera un pdf con las preferencias
def GeneraDocumento(Conexion):
    # Preparamos el pdf
    Pdf = canvas.Canvas(Documento, pagesize=A4)
    Pdf.setAuthor('MaxiFeli')
    Pdf.setTitle('Maximizador de felicidad')

    # Guardamos las dimensiones de la página
    Anchura, Altura = A4

    # Escribimos el título
    Pdf.setFont ('Negrita', TamanoGrande)
    Linea = Altura - Margen
    Pdf.drawCentredString(Anchura/2, Linea, Descripcion)
    Linea -= Interlineado

    # El diccionario de preferencias
    DiccionarioPreferencias = LeePreferencias(Conexion)

    # Montamos una tabla: en cada fila un nombre, en cada columna
    # su preferencia por otro nombre
    DatoTabla = []

    # La primera fila es una cabecera
    Fila = []
    Fila.append('N')
    Fila.append('Nombre')
    for i in range(1, len(ListaNombreUsuario)+1):
        Texto = str(i) if i>9 else ' '+str(i)
        Fila.append(Texto)
    DatoTabla.append(Fila)

    # Las demás filas son las de cada nombre
    for i, NombreUsuario in enumerate(ListaNombreUsuario):
        Nombre = NombreUsuario[0]
        Activo = NombreUsuario[1]
        Fila = []
        Fila.append(str(i+1))
        Fila.append(Nombre)
        for NombreUsuarioPasivo in ListaNombreUsuario:
            Pasivo = NombreUsuarioPasivo[1]
            Cantidad = DiccionarioPreferencias[(Activo, Pasivo)]
            Fila.append(Cantidad)
        DatoTabla.append(Fila)

    # El tamaño de la letra de la tabla debe permitir que quepa en anchura
    PosX = 551
    TamanoTabla = TamanoPequeno

    # Creamos la tabla y reducimos el tamaño de letra has que la tabla quepa
    while PosX > 550:
      Tabla = Table(DatoTabla)
      Tabla.setStyle(TableStyle([
            ('FONT', (0, 0), (-1, 0), 'Negrita'),
            ('FONT', (0, 1), (-1, -1), 'Regular'),
            ('FONT', (2, 0), (-1, 0), 'Mono'),
            ('FONTSIZE', (0, 0), (-1, -1), TamanoTabla),
            ('ALIGN', (0, 0), (-1, 0), 'CENTER'),
            ('ALIGN', (0,0), (1, -1), 'CENTER'),
            ('ALIGN', (1,1), (1, -1), 'LEFT'),
            ('ALIGN', (2,0), (-1, -1), 'CENTER'),
            ('VALIGN', (0,0), (-1, -1), 'MIDDLE'),
            ('LEFTPADDING', (0,0), (-1, -1), 2),
            ('RIGHTPADDING', (0,0), (-1, -1), 2),
            ('TOPPADDING', (0,0), (-1, -1), 1),
            ('BOTTOMPADDING', (0,0), (-1, -1), 2),
            ('BOX', (0,0), (-1,-1), Grosor, colors.black),
            ('GRID', (0,0), (-1,-1), Grosor, colors.black)
            ]))
      PosX, PosY = Tabla.wrapOn(Pdf, Anchura-2*Margen, Altura)
      TamanoTabla -= 0.2

    # Añadimos la tabla a la página
    Tabla.drawOn(Pdf, Margen, Linea-PosY+0.2*cm)

    # Fin de página
    Pdf.showPage()

    # Escribimos el PDF
    Pdf.save()

#------------------------------------------------
# El programa
#------------------------------------------------

# Informamos al usuario
print 'imprime-preferencias'
print '===================='
print

# Averiguamos la fecha y hora
Descripcion = FechaHora()

# Informamos al usuario
print 'Base de datos:', BaseDato
print 'Momento:', Descripcion

# Conversión de medidas: todas las hemos definido en cm
# pero se usan en puntos tipográficos
Margen *= cm
Interlineado *= cm

# Si las fuentes que cargamos no están completas, que no se queje ReportLab
reportlab.rl_config.warnOnMissingFontGlyphs = 0

# Cargamos las fuentes que vamos a usar
pdfmetrics.registerFont(TTFont('Regular', FuenteRegular))
pdfmetrics.registerFont(TTFont('Negrita', FuenteNegrita))
pdfmetrics.registerFont(TTFont('Mono', FuenteMono))

# Abrimos conexión a la base de datos
print 'Leyendo datos...',
Conexion = apsw.Connection(BaseDato)

# La lista de usuarios y nombres
ListaNombreUsuario = LeeNombreUsuario(Conexion)
print 'hecho.'

# Generamos el PDF
print 'Archivo pdf:', Documento
print 'Generando archivo...',
GeneraDocumento(Conexion)
print 'hecho.'

# Cerramos la conexión
Conexion.close(True)

# Hemos acabado; informamos al usuario
print 'Terminado.'
